@@ -1,16 +1,18 @@
 import json
+import pytest
 
 from aiohttp import ClientSession, DummyCookieJar, TCPConnector, web
 from aiohttp.test_utils import unused_port
 
 from asynctest import TestCase as AsyncTestCase
-from asynctest import create_autospec, mock as async_mock
+from asynctest import mock as async_mock
 
 from ...config.default_context import DefaultContextBuilder
 from ...config.injection_context import InjectionContext
-from ...config.provider import ClassProvider
-from ...core.plugin_registry import PluginRegistry
+from ...core.event_bus import Event
+from ...core.in_memory import InMemoryProfile
 from ...core.protocol_registry import ProtocolRegistry
+from ...core.goal_code_registry import GoalCodeRegistry
 from ...transport.outbound.message import OutboundMessage
 from ...utils.stats import Collector
 from ...utils.task_queue import TaskQueue
@@ -120,17 +122,20 @@ class TestAdminServer(AsyncTestCase):
         collector = Collector()
         context.injector.bind_instance(test_module.Collector, collector)
 
+        profile = InMemoryProfile.test_profile()
+
         self.port = unused_port()
         return AdminServer(
             "0.0.0.0",
             self.port,
             context,
+            profile,
             self.outbound_message_router,
             self.webhook_router,
             conductor_stop=async_mock.CoroutineMock(),
             task_queue=TaskQueue(max_active=4) if task_queue else None,
             conductor_stats=(
-                None if task_queue else async_mock.CoroutineMock(return_value=[1, 2])
+                None if task_queue else async_mock.CoroutineMock(return_value={"a": 1})
             ),
         )
 
@@ -157,10 +162,12 @@ class TestAdminServer(AsyncTestCase):
 
         settings = {
             "admin.admin_insecure_mode": False,
+            "admin.admin_client_max_request_size": 4,
             "admin.admin_api_key": "test-api-key",
         }
         server = self.get_admin_server(settings)
         await server.start()
+        assert server.app._client_max_size == 4 * 1024 * 1024
         with async_mock.patch.object(
             server, "websocket_queues", async_mock.MagicMock()
         ) as mock_wsq:
@@ -176,52 +183,103 @@ class TestAdminServer(AsyncTestCase):
             with self.assertRaises(AdminSetupError):
                 await self.get_admin_server(settings).start()
 
-    async def test_responder_send(self):
-        message = OutboundMessage(payload="{}")
-        server = self.get_admin_server()
-        await server.responder.send_outbound(message)
-        assert self.message_results == [(server.context, message)]
-
-    async def test_responder_webhook(self):
-        server = self.get_admin_server()
-        test_url = "target_url"
-        test_attempts = 99
-        server.add_webhook_target(
-            target_url=test_url,
-            topic_filter=["*"],  # cover vacuous filter
-            max_attempts=test_attempts,
-        )
-        test_topic = "test_topic"
-        test_payload = {"test": "TEST"}
-
-        with async_mock.patch.object(
-            server, "websocket_queues", async_mock.MagicMock()
-        ) as mock_wsq:
-            mock_wsq.values = async_mock.MagicMock(
-                return_value=[
-                    async_mock.MagicMock(
-                        authenticated=True, enqueue=async_mock.CoroutineMock()
-                    )
-                ]
-            )
-
-            await server.responder.send_webhook(test_topic, test_payload)
-            assert self.webhook_results == [
-                (test_topic, test_payload, test_url, test_attempts)
-            ]
-
-        server.remove_webhook_target(target_url=test_url)
-        assert test_url not in server.webhook_targets
-
     async def test_import_routes(self):
         # this test just imports all default admin routes
         # for routes with associated tests, this shouldn't make a difference in coverage
         context = InjectionContext()
         context.injector.bind_instance(ProtocolRegistry, ProtocolRegistry())
+        context.injector.bind_instance(GoalCodeRegistry, GoalCodeRegistry())
         await DefaultContextBuilder().load_plugins(context)
         server = self.get_admin_server({"admin.admin_insecure_mode": True}, context)
         app = await server.make_application()
 
+    async def test_import_routes_multitenant_middleware(self):
+        # imports all default admin routes
+        context = InjectionContext()
+        context.injector.bind_instance(ProtocolRegistry, ProtocolRegistry())
+        context.injector.bind_instance(GoalCodeRegistry, GoalCodeRegistry())
+        context.injector.bind_instance(
+            test_module.BaseMultitenantManager,
+            async_mock.MagicMock(spec=test_module.BaseMultitenantManager),
+        )
+        await DefaultContextBuilder().load_plugins(context)
+        server = self.get_admin_server(
+            {
+                "admin.admin_insecure_mode": False,
+                "admin.admin_api_key": "test-api-key",
+            },
+            context,
+        )
+
+        # cover multitenancy start code
+        app = await server.make_application()
+        app["swagger_dict"] = {}
+        await server.on_startup(app)
+
+        # multitenant authz
+        [mt_authz_middle] = [
+            m for m in app.middlewares if ".check_multitenant_authorization" in str(m)
+        ]
+
+        mock_request = async_mock.MagicMock(
+            method="GET",
+            headers={"Authorization": "Bearer ..."},
+            path="/multitenancy/etc",
+            text=async_mock.CoroutineMock(return_value="abc123"),
+        )
+        with self.assertRaises(test_module.web.HTTPUnauthorized):
+            await mt_authz_middle(mock_request, None)
+
+        mock_request = async_mock.MagicMock(
+            method="GET",
+            headers={},
+            path="/protected/non-multitenancy/non-server",
+            text=async_mock.CoroutineMock(return_value="abc123"),
+        )
+        with self.assertRaises(test_module.web.HTTPUnauthorized):
+            await mt_authz_middle(mock_request, None)
+
+        mock_request = async_mock.MagicMock(
+            method="GET",
+            headers={"Authorization": "Bearer ..."},
+            path="/protected/non-multitenancy/non-server",
+            text=async_mock.CoroutineMock(return_value="abc123"),
+        )
+        mock_handler = async_mock.CoroutineMock()
+        await mt_authz_middle(mock_request, mock_handler)
+        assert mock_handler.called_once_with(mock_request)
+
+        # multitenant setup context exception paths
+        [setup_ctx_middle] = [m for m in app.middlewares if ".setup_context" in str(m)]
+
+        mock_request = async_mock.MagicMock(
+            method="GET",
+            headers={"Authorization": "Non-bearer ..."},
+            path="/protected/non-multitenancy/non-server",
+            text=async_mock.CoroutineMock(return_value="abc123"),
+        )
+        with self.assertRaises(test_module.web.HTTPUnauthorized):
+            await setup_ctx_middle(mock_request, None)
+
+        mock_request = async_mock.MagicMock(
+            method="GET",
+            headers={"Authorization": "Bearer ..."},
+            path="/protected/non-multitenancy/non-server",
+            text=async_mock.CoroutineMock(return_value="abc123"),
+        )
+        with async_mock.patch.object(
+            server.multitenant_manager,
+            "get_profile_for_token",
+            async_mock.CoroutineMock(),
+        ) as mock_get_profile:
+            mock_get_profile.side_effect = [
+                test_module.MultitenantManagerError("corrupt token"),
+                test_module.StorageNotFoundError("out of memory"),
+            ]
+            for i in range(2):
+                with self.assertRaises(test_module.web.HTTPUnauthorized):
+                    await setup_ctx_middle(mock_request, None)
+
     async def test_register_external_plugin_x(self):
         context = InjectionContext()
         context.injector.bind_instance(ProtocolRegistry, ProtocolRegistry())
@@ -281,6 +339,31 @@ class TestAdminServer(AsyncTestCase):
         ) as response:
             assert response.status == 200
 
+        # Make sure that OPTIONS requests used by browsers for CORS
+        # are allowed without a x-api-key even when x-api-key security is enabled
+        async with self.client_session.options(
+            f"http://127.0.0.1:{self.port}/status",
+            headers={
+                "Access-Control-Request-Headers": "x-api-key",
+                "Access-Control-Request-Method": "GET",
+                "Connection": "keep-alive",
+                "Host": f"http://127.0.0.1:{self.port}/status",
+                "Origin": "http://localhost:3000",
+                "Referer": "http://localhost:3000/",
+                "Sec-Fetch-Dest": "empty",
+                "Sec-Fetch-Mode": "cors",
+                "Sec-Fetch-Site": "same-site",
+            },
+        ) as response:
+            assert response.status == 200
+            assert response.headers["Access-Control-Allow-Credentials"] == "true"
+            assert response.headers["Access-Control-Allow-Headers"] == "X-API-KEY"
+            assert response.headers["Access-Control-Allow-Methods"] == "GET"
+            assert (
+                response.headers["Access-Control-Allow-Origin"]
+                == "http://localhost:3000"
+            )
+
         async with self.client_session.ws_connect(
             f"http://127.0.0.1:{self.port}/ws", headers={"x-api-key": "test-api-key"}
         ) as ws:
@@ -289,6 +372,42 @@ class TestAdminServer(AsyncTestCase):
 
         await server.stop()
 
+    async def test_query_config(self):
+        settings = {
+            "admin.admin_insecure_mode": False,
+            "admin.admin_api_key": "test-api-key",
+            "admin.webhook_urls": ["localhost:8123/abc#secret", "localhost:8123/def"],
+            "multitenant.jwt_secret": "abc123",
+            "wallet.key": "abc123",
+            "wallet.rekey": "def456",
+            "wallet.seed": "00000000000000000000000000000000",
+            "wallet.storage.creds": "secret",
+        }
+        server = self.get_admin_server(settings)
+        await server.start()
+
+        async with self.client_session.get(
+            f"http://127.0.0.1:{self.port}/status/config",
+            headers={"x-api-key": "test-api-key"},
+        ) as response:
+            config = json.loads(await response.text())["config"]
+            assert "admin.admin_insecure_mode" in config
+            assert all(
+                k not in config
+                for k in [
+                    "admin.admin_api_key",
+                    "multitenant.jwt_secret",
+                    "wallet.key",
+                    "wallet.rekey",
+                    "wallet.seed",
+                    "wallet.storage_creds",
+                ]
+            )
+            assert config["admin.webhook_urls"] == [
+                "localhost:8123/abc",
+                "localhost:8123/def",
+            ]
+
     async def test_visit_shutting_down(self):
         settings = {
             "admin.admin_insecure_mode": True,
@@ -344,3 +463,39 @@ class TestAdminServer(AsyncTestCase):
         ) as response:
             assert response.status == 503
         await server.stop()
+
+
+@pytest.fixture
+async def server():
+    test_class = TestAdminServer()
+    await test_class.setUp()
+    yield test_class.get_admin_server()
+    await test_class.tearDown()
+
+
+@pytest.mark.asyncio
+@pytest.mark.parametrize(
+    "event_topic, webhook_topic",
+    [("acapy::record::topic", "topic"), ("acapy::record::topic::state", "topic")],
+)
+async def test_on_record_event(server, event_topic, webhook_topic):
+    profile = InMemoryProfile.test_profile()
+    with async_mock.patch.object(
+        server, "send_webhook", async_mock.CoroutineMock()
+    ) as mock_send_webhook:
+        await server._on_record_event(profile, Event(event_topic, None))
+        mock_send_webhook.assert_called_once_with(profile, webhook_topic, None)
+
+
+@pytest.mark.asyncio
+async def test_admin_responder_profile_expired_x():
+    def _smaller_scope():
+        profile = InMemoryProfile.test_profile()
+        return test_module.AdminResponder(profile, None)
+
+    responder = _smaller_scope()
+    with pytest.raises(RuntimeError):
+        await responder.send_outbound(None)
+
+    with pytest.raises(RuntimeError):
+        await responder.send_webhook("test", {})
