@@ -1,8 +1,11 @@
 """Commands module common setup."""
 
 from importlib import import_module
+from os import getenv
 from typing import Sequence
 
+PROG = getenv("ACAPY_COMMAND_NAME", "aca-py")
+
 
 def available_commands():
     """Index available commands."""
@@ -10,6 +13,7 @@ def available_commands():
         {"name": "help", "summary": "Print available commands"},
         {"name": "provision", "summary": "Provision an agent"},
         {"name": "start", "summary": "Start a new agent process"},
+        {"name": "upgrade", "summary": "Start agent upgrade process"},
     ]
 
 
