@@ -8,12 +8,18 @@ lifecycle hook callbacks storing state for message threads, etc.
 import asyncio
 import logging
 import os
+import warnings
+
 from typing import Callable, Coroutine, Union
+import weakref
 
 from aiohttp.web import HTTPException
 
-from ..config.injection_context import InjectionContext
+
+from ..connections.models.conn_record import ConnRecord
+from ..core.profile import Profile
 from ..messaging.agent_message import AgentMessage
+from ..messaging.base_message import BaseMessage
 from ..messaging.error import MessageParseError
 from ..messaging.models.base import BaseModelError
 from ..messaging.request_context import RequestContext
@@ -23,9 +29,10 @@ from ..protocols.connections.v1_0.manager import ConnectionManager
 from ..protocols.problem_report.v1_0.message import ProblemReport
 from ..transport.inbound.message import InboundMessage
 from ..transport.outbound.message import OutboundMessage
+from ..transport.outbound.status import OutboundSendStatus
 from ..utils.stats import Collector
 from ..utils.task_queue import CompletedTask, PendingTask, TaskQueue
-from ..utils.tracing import trace_event, get_timer
+from ..utils.tracing import get_timer, trace_event
 
 from .error import ProtocolMinorVersionNotSupported
 from .protocol_registry import ProtocolRegistry
@@ -33,6 +40,10 @@ from .protocol_registry import ProtocolRegistry
 LOGGER = logging.getLogger(__name__)
 
 
+class ProblemReportParseError(MessageParseError):
+    """Error to raise on failure to parse problem-report message."""
+
+
 class Dispatcher:
     """
     Dispatcher class.
@@ -41,15 +52,15 @@ class Dispatcher:
     to other agents.
     """
 
-    def __init__(self, context: InjectionContext):
+    def __init__(self, profile: Profile):
         """Initialize an instance of Dispatcher."""
-        self.context = context
         self.collector: Collector = None
+        self.profile = profile
         self.task_queue: TaskQueue = None
 
     async def setup(self):
         """Perform async instance setup."""
-        self.collector = await self.context.inject(Collector, required=False)
+        self.collector = self.profile.inject_or(Collector)
         max_active = int(os.getenv("DISPATCHER_MAX_ACTIVE", 50))
         self.task_queue = TaskQueue(
             max_active=max_active, timed=bool(self.collector), trace_fn=self.log_task
@@ -85,18 +96,18 @@ class Dispatcher:
 
     def queue_message(
         self,
+        profile: Profile,
         inbound_message: InboundMessage,
         send_outbound: Coroutine,
-        send_webhook: Coroutine = None,
         complete: Callable = None,
     ) -> PendingTask:
         """
         Add a message to the processing queue for handling.
 
         Args:
+            profile: The profile associated with the inbound message
             inbound_message: The inbound message instance
             send_outbound: Async function to send outbound messages
-            send_webhook: Async function to dispatch a webhook
             complete: Function to call when the handler has completed
 
         Returns:
@@ -104,22 +115,23 @@ class Dispatcher:
 
         """
         return self.put_task(
-            self.handle_message(inbound_message, send_outbound, send_webhook), complete
+            self.handle_message(profile, inbound_message, send_outbound),
+            complete,
         )
 
     async def handle_message(
         self,
+        profile: Profile,
         inbound_message: InboundMessage,
         send_outbound: Coroutine,
-        send_webhook: Coroutine = None,
     ):
         """
         Configure responder and message context and invoke the message handler.
 
         Args:
+            profile: The profile associated with the inbound message
             inbound_message: The inbound message instance
             send_outbound: Async function to send outbound messages
-            send_webhook: Async function to dispatch a webhook
 
         Returns:
             The response from the handler
@@ -127,65 +139,83 @@ class Dispatcher:
         """
         r_time = get_timer()
 
-        connection_mgr = ConnectionManager(self.context)
-        connection = await connection_mgr.find_inbound_connection(
-            inbound_message.receipt
-        )
-        if connection:
-            inbound_message.connection_id = connection.connection_id
-
         error_result = None
+        message = None
         try:
             message = await self.make_message(inbound_message.payload)
+        except ProblemReportParseError:
+            pass  # avoid problem report recursion
         except MessageParseError as e:
             LOGGER.error(f"Message parsing failed: {str(e)}, sending problem report")
-            error_result = ProblemReport(explain_ltxt=str(e))
+            error_result = ProblemReport(
+                description={
+                    "en": str(e),
+                    "code": "message-parse-failure",
+                }
+            )
             if inbound_message.receipt.thread_id:
                 error_result.assign_thread_id(inbound_message.receipt.thread_id)
-            message = None
 
         trace_event(
-            self.context.settings,
+            self.profile.settings,
             message,
             outcome="Dispatcher.handle_message.START",
         )
 
-        context = RequestContext(base_context=self.context)
+        context = RequestContext(profile)
         context.message = message
         context.message_receipt = inbound_message.receipt
-        context.connection_ready = connection and connection.is_ready
-        context.connection_record = connection
 
         responder = DispatcherResponder(
             context,
             inbound_message,
             send_outbound,
-            send_webhook,
-            connection_id=connection and connection.connection_id,
             reply_session_id=inbound_message.session_id,
             reply_to_verkey=inbound_message.receipt.sender_verkey,
         )
 
-        if error_result:
-            await responder.send_reply(error_result)
-            return
-
         context.injector.bind_instance(BaseResponder, responder)
 
-        handler_cls = context.message.Handler
-        handler = handler_cls().handle
-        if self.collector:
-            handler = self.collector.wrap_coro(handler, [handler.__qualname__])
-        await handler(context, responder)
+        # When processing oob attach message we supply the connection id
+        # associated with the inbound message
+        if inbound_message.connection_id:
+            async with self.profile.session() as session:
+                connection = await ConnRecord.retrieve_by_id(
+                    session, inbound_message.connection_id
+                )
+        else:
+            connection_mgr = ConnectionManager(profile)
+            connection = await connection_mgr.find_inbound_connection(
+                inbound_message.receipt
+            )
+            del connection_mgr
+
+        if connection:
+            inbound_message.connection_id = connection.connection_id
+
+        context.connection_ready = connection and connection.is_ready
+        context.connection_record = connection
+        responder.connection_id = connection and connection.connection_id
+
+        if error_result:
+            await responder.send_reply(error_result)
+        elif context.message:
+            context.injector.bind_instance(BaseResponder, responder)
+
+            handler_cls = context.message.Handler
+            handler = handler_cls().handle
+            if self.collector:
+                handler = self.collector.wrap_coro(handler, [handler.__qualname__])
+            await handler(context, responder)
 
         trace_event(
-            self.context.settings,
+            self.profile.settings,
             context.message,
             outcome="Dispatcher.handle_message.END",
             perf_counter=r_time,
         )
 
-    async def make_message(self, parsed_msg: dict) -> AgentMessage:
+    async def make_message(self, parsed_msg: dict) -> BaseMessage:
         """
         Deserialize a message dict into the appropriate message instance.
 
@@ -204,13 +234,14 @@ class Dispatcher:
             the given type
 
         """
-
-        registry: ProtocolRegistry = await self.context.inject(ProtocolRegistry)
+        if not isinstance(parsed_msg, dict):
+            raise MessageParseError("Expected a JSON object")
         message_type = parsed_msg.get("@type")
 
         if not message_type:
             raise MessageParseError("Message does not contain '@type' parameter")
 
+        registry: ProtocolRegistry = self.profile.inject(ProtocolRegistry)
         try:
             message_cls = registry.resolve_message_class(message_type)
         except ProtocolMinorVersionNotSupported as e:
@@ -222,6 +253,8 @@ class Dispatcher:
         try:
             instance = message_cls.deserialize(parsed_msg)
         except BaseModelError as e:
+            if "/problem-report" in message_type:
+                raise ProblemReportParseError("Error parsing problem report message")
             raise MessageParseError(f"Error deserializing message: {e}") from e
 
         return instance
@@ -239,7 +272,6 @@ class DispatcherResponder(BaseResponder):
         context: RequestContext,
         inbound_message: InboundMessage,
         send_outbound: Coroutine,
-        send_webhook: Coroutine = None,
         **kwargs,
     ):
         """
@@ -249,17 +281,18 @@ class DispatcherResponder(BaseResponder):
             context: The request context of the incoming message
             inbound_message: The inbound message triggering this handler
             send_outbound: Async function to send outbound message
-            send_webhook: Async function to dispatch a webhook
 
         """
         super().__init__(**kwargs)
-        self._context = context
+        # Weakly hold the context so it can be properly garbage collected.
+        # Binding this DispatcherResponder into the context creates a circular
+        # reference.
+        self._context = weakref.ref(context)
         self._inbound_message = inbound_message
         self._send = send_outbound
-        self._webhook = send_webhook
 
     async def create_outbound(
-        self, message: Union[AgentMessage, str, bytes], **kwargs
+        self, message: Union[AgentMessage, BaseMessage, str, bytes], **kwargs
     ) -> OutboundMessage:
         """
         Create an OutboundMessage from a message body.
@@ -267,36 +300,48 @@ class DispatcherResponder(BaseResponder):
         Args:
             message: The message payload
         """
-        if isinstance(message, AgentMessage) and self._context.settings.get(
-            "timing.enabled"
-        ):
+        context = self._context()
+        if not context:
+            raise RuntimeError("weakref to context has expired")
+
+        if isinstance(message, AgentMessage) and context.settings.get("timing.enabled"):
             # Inject the timing decorator
-            in_time = (
-                self._context.message_receipt and self._context.message_receipt.in_time
-            )
+            in_time = context.message_receipt and context.message_receipt.in_time
             if not message._decorators.get("timing"):
                 message._decorators["timing"] = {
                     "in_time": in_time,
                     "out_time": datetime_now(),
                 }
+
         return await super().create_outbound(message, **kwargs)
 
-    async def send_outbound(self, message: OutboundMessage):
+    async def send_outbound(self, message: OutboundMessage) -> OutboundSendStatus:
         """
         Send outbound message.
 
         Args:
             message: The `OutboundMessage` to be sent
         """
-        await self._send(self._context, message, self._inbound_message)
+        context = self._context()
+        if not context:
+            raise RuntimeError("weakref to context has expired")
+
+        return await self._send(context.profile, message, self._inbound_message)
 
     async def send_webhook(self, topic: str, payload: dict):
         """
-        Dispatch a webhook.
+        Dispatch a webhook. DEPRECATED: use the event bus instead.
 
         Args:
             topic: the webhook topic identifier
             payload: the webhook payload value
         """
-        if self._webhook:
-            await self._webhook(topic, payload)
+        warnings.warn(
+            "responder.send_webhook is deprecated; please use the event bus instead.",
+            DeprecationWarning,
+        )
+        context = self._context()
+        if not context:
+            raise RuntimeError("weakref to context has expired")
+
+        await context.profile.notify("acapy::webhook::" + topic, payload)
