@@ -1,35 +1,37 @@
-import asyncio
 import json
 
-from asynctest import TestCase as AsyncTestCase, mock as async_mock
+import pytest
 
+from asynctest import TestCase as AsyncTestCase, mock as async_mock
 from marshmallow import EXCLUDE
 
 from ...config.injection_context import InjectionContext
-from ...connections.models.connection_record import ConnectionRecord
+from ...core.event_bus import EventBus
+from ...core.in_memory import InMemoryProfile
+from ...core.profile import Profile
 from ...core.protocol_registry import ProtocolRegistry
 from ...messaging.agent_message import AgentMessage, AgentMessageSchema
-from ...messaging.responder import MockResponder
-from ...messaging.util import datetime_now
-
+from ...messaging.request_context import RequestContext
 from ...protocols.didcomm_prefix import DIDCommPrefix
+from ...protocols.issue_credential.v2_0.message_types import CRED_20_PROBLEM_REPORT
+from ...protocols.issue_credential.v2_0.messages.cred_problem_report import (
+    V20CredProblemReport,
+)
 from ...protocols.problem_report.v1_0.message import ProblemReport
-
 from ...transport.inbound.message import InboundMessage
 from ...transport.inbound.receipt import MessageReceipt
 from ...transport.outbound.message import OutboundMessage
+from ...utils.stats import Collector
 
 from .. import dispatcher as test_module
 
 
-def make_context() -> InjectionContext:
-    context = InjectionContext()
-    context.injector.bind_instance(ProtocolRegistry, ProtocolRegistry())
-
-    collector = test_module.Collector()
-    context.injector.bind_instance(test_module.Collector, collector)
-
-    return context
+def make_profile() -> Profile:
+    profile = InMemoryProfile.test_profile()
+    profile.context.injector.bind_instance(ProtocolRegistry, ProtocolRegistry())
+    profile.context.injector.bind_instance(Collector, Collector())
+    profile.context.injector.bind_instance(EventBus, EventBus())
+    return profile
 
 
 def make_inbound(payload) -> InboundMessage:
@@ -87,16 +89,15 @@ class StubV1_2AgentMessageHandler:
 
 class TestDispatcher(AsyncTestCase):
     async def test_dispatch(self):
-        context = make_context()
-        context.enforce_typing = False
-        registry = await context.inject(ProtocolRegistry)
+        profile = make_profile()
+        registry = profile.inject(ProtocolRegistry)
         registry.register_message_types(
             {
                 pfx.qualify(StubAgentMessage.Meta.message_type): StubAgentMessage
                 for pfx in DIDCommPrefix
             }
         )
-        dispatcher = test_module.Dispatcher(context)
+        dispatcher = test_module.Dispatcher(profile)
         await dispatcher.setup()
         rcv = Receiver()
         message = {
@@ -113,7 +114,9 @@ class TestDispatcher(AsyncTestCase):
                     return_value=async_mock.MagicMock(connection_id="dummy")
                 )
             )
-            await dispatcher.queue_message(make_inbound(message), rcv.send)
+            await dispatcher.queue_message(
+                dispatcher.profile, make_inbound(message), rcv.send
+            )
             await dispatcher.task_queue
             handler_mock.assert_awaited_once()
             assert isinstance(handler_mock.call_args[0][1].message, StubAgentMessage)
@@ -122,9 +125,8 @@ class TestDispatcher(AsyncTestCase):
             )
 
     async def test_dispatch_versioned_message(self):
-        context = make_context()
-        context.enforce_typing = False
-        registry = await context.inject(ProtocolRegistry)
+        profile = make_profile()
+        registry = profile.inject(ProtocolRegistry)
         registry.register_message_types(
             {
                 DIDCommPrefix.qualify_current(
@@ -138,7 +140,7 @@ class TestDispatcher(AsyncTestCase):
                 "path": "v1_1",
             },
         )
-        dispatcher = test_module.Dispatcher(context)
+        dispatcher = test_module.Dispatcher(profile)
         await dispatcher.setup()
         rcv = Receiver()
         message = {
@@ -148,7 +150,9 @@ class TestDispatcher(AsyncTestCase):
         with async_mock.patch.object(
             StubAgentMessageHandler, "handle", autospec=True
         ) as handler_mock:
-            await dispatcher.queue_message(make_inbound(message), rcv.send)
+            await dispatcher.queue_message(
+                dispatcher.profile, make_inbound(message), rcv.send
+            )
             await dispatcher.task_queue
             handler_mock.assert_awaited_once()
             assert isinstance(handler_mock.call_args[0][1].message, StubAgentMessage)
@@ -157,9 +161,8 @@ class TestDispatcher(AsyncTestCase):
             )
 
     async def test_dispatch_versioned_message_no_message_class(self):
-        context = make_context()
-        context.enforce_typing = False
-        registry = await context.inject(ProtocolRegistry)
+        profile = make_profile()
+        registry = profile.inject(ProtocolRegistry)
         registry.register_message_types(
             {
                 DIDCommPrefix.qualify_current(
@@ -173,7 +176,7 @@ class TestDispatcher(AsyncTestCase):
                 "path": "v1_1",
             },
         )
-        dispatcher = test_module.Dispatcher(context)
+        dispatcher = test_module.Dispatcher(profile)
         await dispatcher.setup()
         rcv = Receiver()
         message = {"@type": "proto-name/1.1/no-such-message-type"}
@@ -181,7 +184,9 @@ class TestDispatcher(AsyncTestCase):
         with async_mock.patch.object(
             StubAgentMessageHandler, "handle", autospec=True
         ) as handler_mock:
-            await dispatcher.queue_message(make_inbound(message), rcv.send)
+            await dispatcher.queue_message(
+                dispatcher.profile, make_inbound(message), rcv.send
+            )
             await dispatcher.task_queue
             assert rcv.messages and isinstance(rcv.messages[0][1], OutboundMessage)
             payload = json.loads(rcv.messages[0][1].payload)
@@ -190,9 +195,8 @@ class TestDispatcher(AsyncTestCase):
             )
 
     async def test_dispatch_versioned_message_message_class_deserialize_x(self):
-        context = make_context()
-        context.enforce_typing = False
-        registry = await context.inject(ProtocolRegistry)
+        profile = make_profile()
+        registry = profile.inject(ProtocolRegistry)
         registry.register_message_types(
             {
                 DIDCommPrefix.qualify_current(
@@ -206,7 +210,7 @@ class TestDispatcher(AsyncTestCase):
                 "path": "v1_1",
             },
         )
-        dispatcher = test_module.Dispatcher(context)
+        dispatcher = test_module.Dispatcher(profile)
         await dispatcher.setup()
         rcv = Receiver()
         message = {"@type": "proto-name/1.1/no-such-message-type"}
@@ -221,7 +225,9 @@ class TestDispatcher(AsyncTestCase):
                     side_effect=test_module.BaseModelError()
                 )
             )
-            await dispatcher.queue_message(make_inbound(message), rcv.send)
+            await dispatcher.queue_message(
+                dispatcher.profile, make_inbound(message), rcv.send
+            )
             await dispatcher.task_queue
             assert rcv.messages and isinstance(rcv.messages[0][1], OutboundMessage)
             payload = json.loads(rcv.messages[0][1].payload)
@@ -230,9 +236,8 @@ class TestDispatcher(AsyncTestCase):
             )
 
     async def test_dispatch_versioned_message_handle_greater_succeeds(self):
-        context = make_context()
-        context.enforce_typing = False
-        registry = await context.inject(ProtocolRegistry)
+        profile = make_profile()
+        registry = profile.inject(ProtocolRegistry)
         registry.register_message_types(
             {
                 DIDCommPrefix.qualify_current(
@@ -246,7 +251,7 @@ class TestDispatcher(AsyncTestCase):
                 "path": "v1_1",
             },
         )
-        dispatcher = test_module.Dispatcher(context)
+        dispatcher = test_module.Dispatcher(profile)
         await dispatcher.setup()
         rcv = Receiver()
         message = {
@@ -258,7 +263,9 @@ class TestDispatcher(AsyncTestCase):
         with async_mock.patch.object(
             StubAgentMessageHandler, "handle", autospec=True
         ) as handler_mock:
-            await dispatcher.queue_message(make_inbound(message), rcv.send)
+            await dispatcher.queue_message(
+                dispatcher.profile, make_inbound(message), rcv.send
+            )
             await dispatcher.task_queue
             handler_mock.assert_awaited_once()
             assert isinstance(handler_mock.call_args[0][1].message, StubAgentMessage)
@@ -267,9 +274,8 @@ class TestDispatcher(AsyncTestCase):
             )
 
     async def test_dispatch_versioned_message_fail(self):
-        context = make_context()
-        context.enforce_typing = False
-        registry = await context.inject(ProtocolRegistry)
+        profile = make_profile()
+        registry = profile.inject(ProtocolRegistry)
         registry.register_message_types(
             {
                 DIDCommPrefix.qualify_current(
@@ -283,7 +289,7 @@ class TestDispatcher(AsyncTestCase):
                 "path": "v1_2",
             },
         )
-        dispatcher = test_module.Dispatcher(context)
+        dispatcher = test_module.Dispatcher(profile)
         await dispatcher.setup()
         rcv = Receiver()
         message = {
@@ -293,7 +299,9 @@ class TestDispatcher(AsyncTestCase):
         with async_mock.patch.object(
             StubAgentMessageHandler, "handle", autospec=True
         ) as handler_mock:
-            await dispatcher.queue_message(make_inbound(message), rcv.send)
+            await dispatcher.queue_message(
+                dispatcher.profile, make_inbound(message), rcv.send
+            )
             await dispatcher.task_queue
             assert rcv.messages and isinstance(rcv.messages[0][1], OutboundMessage)
             payload = json.loads(rcv.messages[0][1].payload)
@@ -301,23 +309,48 @@ class TestDispatcher(AsyncTestCase):
                 ProblemReport.Meta.message_type
             )
 
-    async def test_bad_message_dispatch(self):
-        dispatcher = test_module.Dispatcher(make_context())
+    async def test_bad_message_dispatch_parse_x(self):
+        dispatcher = test_module.Dispatcher(make_profile())
         await dispatcher.setup()
         rcv = Receiver()
-        bad_message = {"bad": "message"}
-        await dispatcher.queue_message(make_inbound(bad_message), rcv.send)
-        await dispatcher.task_queue
-        assert rcv.messages and isinstance(rcv.messages[0][1], OutboundMessage)
-        payload = json.loads(rcv.messages[0][1].payload)
-        assert payload["@type"] == DIDCommPrefix.qualify_current(
-            ProblemReport.Meta.message_type
+        bad_messages = ["not even a dict", {"bad": "message"}]
+        for bad in bad_messages:
+            await dispatcher.queue_message(
+                dispatcher.profile, make_inbound(bad), rcv.send
+            )
+            await dispatcher.task_queue
+            assert rcv.messages and isinstance(rcv.messages[0][1], OutboundMessage)
+            payload = json.loads(rcv.messages[0][1].payload)
+            assert payload["@type"] == DIDCommPrefix.qualify_current(
+                ProblemReport.Meta.message_type
+            )
+            rcv.messages.clear()
+
+    async def test_bad_message_dispatch_problem_report_x(self):
+        profile = make_profile()
+        registry = profile.inject(ProtocolRegistry)
+        registry.register_message_types(
+            {
+                pfx.qualify(CRED_20_PROBLEM_REPORT): V20CredProblemReport
+                for pfx in DIDCommPrefix
+            }
         )
+        dispatcher = test_module.Dispatcher(profile)
+        await dispatcher.setup()
+        rcv = Receiver()
+        bad_message = {
+            "@type": DIDCommPrefix.qualify_current(CRED_20_PROBLEM_REPORT),
+            "description": "should be a dict",
+        }
+        await dispatcher.queue_message(
+            dispatcher.profile, make_inbound(bad_message), rcv.send
+        )
+        await dispatcher.task_queue
+        assert not rcv.messages
 
     async def test_dispatch_log(self):
-        context = make_context()
-        context.enforce_typing = False
-        registry = await context.inject(ProtocolRegistry)
+        profile = make_profile()
+        registry = profile.inject(ProtocolRegistry)
         registry.register_message_types(
             {
                 DIDCommPrefix.qualify_current(
@@ -326,7 +359,7 @@ class TestDispatcher(AsyncTestCase):
             },
         )
 
-        dispatcher = test_module.Dispatcher(context)
+        dispatcher = test_module.Dispatcher(profile)
         await dispatcher.setup()
 
         mock_task = async_mock.MagicMock(
@@ -341,35 +374,50 @@ class TestDispatcher(AsyncTestCase):
         )
         dispatcher.log_task(mock_task)
 
-    async def test_create_outbound_send_webhook(self):
-        context = make_context()
-        context.message_receipt = async_mock.MagicMock(in_time=datetime_now())
-        context.settings = {"timing.enabled": True}
-        message = StubAgentMessage()
-        responder = test_module.DispatcherResponder(
-            context, message, None, async_mock.CoroutineMock()
-        )
-        result = await responder.create_outbound(message)
-        assert json.loads(result.payload)["@type"] == DIDCommPrefix.qualify_current(
-            StubAgentMessage.Meta.message_type
-        )
-        await responder.send_webhook("topic", "payload")
-
     async def test_create_send_outbound(self):
+        profile = make_profile()
+        context = RequestContext(
+            profile,
+            settings={"timing.enabled": True},
+        )
         message = StubAgentMessage()
-        responder = MockResponder()
+        responder = test_module.DispatcherResponder(context, message, None)
         outbound_message = await responder.create_outbound(message)
-        await responder.send_outbound(outbound_message)
-        assert len(responder.messages) == 1
+        with async_mock.patch.object(responder, "_send", async_mock.CoroutineMock()):
+            await responder.send_outbound(outbound_message)
+
+    async def test_create_send_webhook(self):
+        profile = make_profile()
+        context = RequestContext(profile)
+        message = StubAgentMessage()
+        responder = test_module.DispatcherResponder(context, message, None)
+        with pytest.deprecated_call():
+            await responder.send_webhook("topic", {"pay": "load"})
 
     async def test_create_enc_outbound(self):
-        context = make_context()
+        profile = make_profile()
+        context = RequestContext(profile)
         message = b"abc123xyz7890000"
-        responder = test_module.DispatcherResponder(
-            context, message, None, async_mock.CoroutineMock()
-        )
+        responder = test_module.DispatcherResponder(context, message, None)
         with async_mock.patch.object(
             responder, "send_outbound", async_mock.CoroutineMock()
         ) as mock_send_outbound:
             await responder.send(message)
             assert mock_send_outbound.called_once()
+
+    async def test_expired_context_x(self):
+        def _smaller_scope():
+            profile = make_profile()
+            context = RequestContext(profile)
+            message = b"abc123xyz7890000"
+            return test_module.DispatcherResponder(context, message, None)
+
+        responder = _smaller_scope()
+        with self.assertRaises(RuntimeError):
+            await responder.create_outbound(b"test")
+
+        with self.assertRaises(RuntimeError):
+            await responder.send_outbound(None)
+
+        with self.assertRaises(RuntimeError):
+            await responder.send_webhook("test", {})
