@@ -43,7 +43,7 @@ class ConnectionInvitation(AgentMessage):
         Initialize connection invitation object.
 
         Args:
-            label: Optional label for connection
+            label: Optional label for connection invitation
             did: DID for this connection invitation
             recipient_keys: List of recipient keys
             endpoint: Endpoint which this agent can be reached at
@@ -56,6 +56,7 @@ class ConnectionInvitation(AgentMessage):
         self.recipient_keys = list(recipient_keys) if recipient_keys else None
         self.endpoint = endpoint
         self.routing_keys = list(routing_keys) if routing_keys else None
+        self.image_url = image_url
 
     def to_url(self, base_url: str = None) -> str:
         """
@@ -100,7 +101,9 @@ class ConnectionInvitationSchema(AgentMessageSchema):
         unknown = EXCLUDE
 
     label = fields.Str(
-        required=False, description="Optional label for connection", example="Bob"
+        required=False,
+        description="Optional label for connection invitation",
+        example="Bob",
     )
     did = fields.Str(
         required=False, description="DID for connection invitation", **INDY_DID
