@@ -2,16 +2,16 @@ from unittest import mock, TestCase
 
 from asynctest import TestCase as AsyncTestCase
 
+from ......wallet.key_type import KeyType
 from ......connections.models.diddoc import (
     DIDDoc,
     PublicKey,
     PublicKeyType,
     Service,
 )
-from ......wallet.basic import BasicWallet
+from ......core.in_memory import InMemoryProfile
 
 from .....didcomm_prefix import DIDCommPrefix
-
 from ...message_types import CONNECTION_RESPONSE
 from ...models.connection_detail import ConnectionDetail
 
@@ -106,8 +106,9 @@ class TestConnectionResponseSchema(AsyncTestCase, TestConfig):
         connection_response = ConnectionResponse(
             connection=ConnectionDetail(did=self.test_did, did_doc=self.make_did_doc())
         )
-        wallet = BasicWallet()
-        key_info = await wallet.create_signing_key()
+        session = InMemoryProfile.test_session()
+        wallet = session.wallet
+        key_info = await wallet.create_signing_key(KeyType.ED25519)
         await connection_response.sign_field("connection", key_info.verkey, wallet)
         data = connection_response.serialize()
         model_instance = ConnectionResponse.deserialize(data)
