@@ -13,10 +13,8 @@ from aiohttp_apispec import (
 
 from marshmallow import fields, validate, validates_schema
 
-from ....connections.models.connection_record import (
-    ConnectionRecord,
-    ConnectionRecordSchema,
-)
+from ....admin.request_context import AdminRequestContext
+from ....connections.models.conn_record import ConnRecord, ConnRecordSchema
 from ....messaging.models.base import BaseModelError
 from ....messaging.models.openapi import OpenAPISchema
 from ....messaging.valid import (
@@ -36,21 +34,89 @@ from .messages.connection_invitation import (
 )
 
 
+class ConnectionModuleResponseSchema(OpenAPISchema):
+    """Response schema for connection module."""
+
+
 class ConnectionListSchema(OpenAPISchema):
     """Result schema for connection list."""
 
     results = fields.List(
-        fields.Nested(ConnectionRecordSchema()),
+        fields.Nested(ConnRecordSchema()),
         description="List of connection records",
     )
 
 
+class ConnectionMetadataSchema(OpenAPISchema):
+    """Result schema for connection metadata."""
+
+    results = fields.Dict(
+        description="Dictionary of metadata associated with connection.",
+    )
+
+
+class ConnectionMetadataSetRequestSchema(OpenAPISchema):
+    """Request Schema for set metadata."""
+
+    metadata = fields.Dict(
+        required=True,
+        description="Dictionary of metadata to set for connection.",
+    )
+
+
+class ConnectionMetadataQuerySchema(OpenAPISchema):
+    """Query schema for metadata."""
+
+    key = fields.Str(required=False, description="Key to retrieve.")
+
+
 class ReceiveInvitationRequestSchema(ConnectionInvitationSchema):
     """Request schema for receive invitation request."""
 
     @validates_schema
     def validate_fields(self, data, **kwargs):
-        """Bypass middleware field validation."""
+        """Bypass middleware field validation: marshmallow has no data yet."""
+
+
+MEDIATION_ID_SCHEMA = {
+    "validate": UUIDFour(),
+    "example": UUIDFour.EXAMPLE,
+}
+
+
+class CreateInvitationRequestSchema(OpenAPISchema):
+    """Request schema for invitation connection target."""
+
+    recipient_keys = fields.List(
+        fields.Str(description="Recipient public key", **INDY_RAW_PUBLIC_KEY),
+        required=False,
+        description="List of recipient keys",
+    )
+    service_endpoint = fields.Str(
+        required=False,
+        description="Connection endpoint",
+        example="http://192.168.56.102:8020",
+    )
+    routing_keys = fields.List(
+        fields.Str(description="Routing key", **INDY_RAW_PUBLIC_KEY),
+        required=False,
+        description="List of routing keys",
+    )
+    my_label = fields.Str(
+        description="Optional label for connection invitation",
+        required=False,
+        example="Bob",
+    )
+    metadata = fields.Dict(
+        description="Optional metadata to attach to the connection created with "
+        "the invitation",
+        required=False,
+    )
+    mediation_id = fields.Str(
+        required=False,
+        description="Identifier for active mediation record to be used",
+        **MEDIATION_ID_SCHEMA
+    )
 
 
 class InvitationResultSchema(OpenAPISchema):
@@ -77,15 +143,10 @@ class ConnectionStaticRequestSchema(OpenAPISchema):
     their_did = fields.Str(description="Remote DID", required=False, **INDY_DID)
     their_verkey = fields.Str(description="Remote verification key", required=False)
     their_endpoint = fields.Str(
-        description="URL endpoint for the other party", required=False, **ENDPOINT
-    )
-    their_role = fields.Str(
-        description="Role to assign to this connection",
-        required=False,
-        example="Point of contact",
+        description="URL endpoint for other party", required=False, **ENDPOINT
     )
     their_label = fields.Str(
-        description="Label to assign to this connection", required=False
+        description="Other party's label for this connection", required=False
     )
     alias = fields.Str(description="Alias to assign to this connection", required=False)
 
@@ -94,7 +155,7 @@ class ConnectionStaticResultSchema(OpenAPISchema):
     """Result schema for new static connection."""
 
     my_did = fields.Str(description="Local DID", required=True, **INDY_DID)
-    mv_verkey = fields.Str(
+    my_verkey = fields.Str(
         description="My verification key", required=True, **INDY_RAW_PUBLIC_KEY
     )
     my_endpoint = fields.Str(description="My URL endpoint", required=True, **ENDPOINT)
@@ -102,7 +163,7 @@ class ConnectionStaticResultSchema(OpenAPISchema):
     their_verkey = fields.Str(
         description="Remote verification key", required=True, **INDY_RAW_PUBLIC_KEY
     )
-    record = fields.Nested(ConnectionRecordSchema, required=True)
+    record = fields.Nested(ConnRecordSchema, required=True)
 
 
 class ConnectionsListQueryStringSchema(OpenAPISchema):
@@ -113,11 +174,6 @@ class ConnectionsListQueryStringSchema(OpenAPISchema):
         required=False,
         example="Barry",
     )
-    initiator = fields.Str(
-        description="Connection initiator",
-        required=False,
-        validate=validate.OneOf(["self", "external"]),
-    )
     invitation_key = fields.Str(
         description="invitation key", required=False, **INDY_RAW_PUBLIC_KEY
     )
@@ -126,18 +182,33 @@ class ConnectionsListQueryStringSchema(OpenAPISchema):
         description="Connection state",
         required=False,
         validate=validate.OneOf(
-            [
-                getattr(ConnectionRecord, m)
-                for m in vars(ConnectionRecord)
-                if m.startswith("STATE_")
-            ]
+            {label for state in ConnRecord.State for label in state.value}
         ),
     )
     their_did = fields.Str(description="Their DID", required=False, **INDY_DID)
+    their_public_did = fields.Str(
+        description="Their Public DID", required=False, **INDY_DID
+    )
     their_role = fields.Str(
-        description="Their assigned connection role",
+        description="Their role in the connection protocol",
         required=False,
-        example="Point of contact",
+        validate=validate.OneOf(
+            [label for role in ConnRecord.Role for label in role.value]
+        ),
+        example=ConnRecord.Role.REQUESTER.rfc160,
+    )
+    connection_protocol = fields.Str(
+        description="Connection protocol used",
+        required=False,
+        validate=validate.OneOf(
+            [proto.aries_protocol for proto in ConnRecord.Protocol]
+        ),
+        example=ConnRecord.Protocol.RFC_0160.aries_protocol,
+    )
+    invitation_msg_id = fields.UUID(
+        description="Identifier of the associated Invitation Mesage",
+        required=False,
+        example=UUIDFour.EXAMPLE,
     )
 
 
@@ -150,7 +221,7 @@ class CreateInvitationQueryStringSchema(OpenAPISchema):
         example="Barry",
     )
     auto_accept = fields.Boolean(
-        description="Auto-accept connection (default as per configuration)",
+        description="Auto-accept connection (defaults to configuration)",
         required=False,
     )
     public = fields.Boolean(
@@ -173,6 +244,11 @@ class ReceiveInvitationQueryStringSchema(OpenAPISchema):
         description="Auto-accept connection (defaults to configuration)",
         required=False,
     )
+    mediation_id = fields.Str(
+        required=False,
+        description="Identifier for active mediation record to be used",
+        **MEDIATION_ID_SCHEMA
+    )
 
 
 class AcceptInvitationQueryStringSchema(OpenAPISchema):
@@ -182,6 +258,11 @@ class AcceptInvitationQueryStringSchema(OpenAPISchema):
     my_label = fields.Str(
         description="Label for connection", required=False, example="Broker"
     )
+    mediation_id = fields.Str(
+        required=False,
+        description="Identifier for active mediation record to be used",
+        **MEDIATION_ID_SCHEMA
+    )
 
 
 class AcceptRequestQueryStringSchema(OpenAPISchema):
@@ -190,7 +271,7 @@ class AcceptRequestQueryStringSchema(OpenAPISchema):
     my_endpoint = fields.Str(description="My URL endpoint", required=False, **ENDPOINT)
 
 
-class ConnIdMatchInfoSchema(OpenAPISchema):
+class ConnectionsConnIdMatchInfoSchema(OpenAPISchema):
     """Path parameters and validators for request taking connection id."""
 
     conn_id = fields.Str(
@@ -212,14 +293,24 @@ class ConnIdRefIdMatchInfoSchema(OpenAPISchema):
     )
 
 
+class EndpointsResultSchema(OpenAPISchema):
+    """Result schema for connection endpoints."""
+
+    my_endpoint = fields.Str(description="My endpoint", **ENDPOINT)
+    their_endpoint = fields.Str(description="Their endpoint", **ENDPOINT)
+
+
 def connection_sort_key(conn):
     """Get the sorting key for a particular connection."""
-    if conn["state"] == ConnectionRecord.STATE_INACTIVE:
+
+    conn_rec_state = ConnRecord.State.get(conn["state"])
+    if conn_rec_state is ConnRecord.State.ABANDONED:
         pfx = "2"
-    elif conn["state"] == ConnectionRecord.STATE_INVITATION:
+    elif conn_rec_state is ConnRecord.State.INVITATION:
         pfx = "1"
     else:
         pfx = "0"
+
     return pfx + conn["created_at"]
 
 
@@ -228,7 +319,7 @@ def connection_sort_key(conn):
     summary="Query agent-to-agent connections",
 )
 @querystring_schema(ConnectionsListQueryStringSchema())
-@response_schema(ConnectionListSchema(), 200)
+@response_schema(ConnectionListSchema(), 200, description="")
 async def connections_list(request: web.BaseRequest):
     """
     Request handler for searching connection records.
@@ -240,37 +331,52 @@ async def connections_list(request: web.BaseRequest):
         The connection list response
 
     """
-    context = request.app["request_context"]
+    context: AdminRequestContext = request["context"]
+
     tag_filter = {}
     for param_name in (
         "invitation_id",
         "my_did",
         "their_did",
         "request_id",
+        "invitation_key",
+        "their_public_did",
+        "invitation_msg_id",
     ):
         if param_name in request.query and request.query[param_name] != "":
             tag_filter[param_name] = request.query[param_name]
+
     post_filter = {}
-    for param_name in (
-        "alias",
-        "initiator",
-        "state",
-        "their_role",
-    ):
-        if param_name in request.query and request.query[param_name] != "":
-            post_filter[param_name] = request.query[param_name]
+    if request.query.get("alias"):
+        post_filter["alias"] = request.query["alias"]
+    if request.query.get("state"):
+        post_filter["state"] = [
+            v for v in ConnRecord.State.get(request.query["state"]).value
+        ]
+    if request.query.get("their_role"):
+        post_filter["their_role"] = [
+            v for v in ConnRecord.Role.get(request.query["their_role"]).value
+        ]
+    if request.query.get("connection_protocol"):
+        post_filter["connection_protocol"] = request.query["connection_protocol"]
+
+    profile = context.profile
     try:
-        records = await ConnectionRecord.query(context, tag_filter, post_filter)
+        async with profile.session() as session:
+            records = await ConnRecord.query(
+                session, tag_filter, post_filter_positive=post_filter, alt=True
+            )
         results = [record.serialize() for record in records]
         results.sort(key=connection_sort_key)
     except (StorageError, BaseModelError) as err:
         raise web.HTTPBadRequest(reason=err.roll_up) from err
+
     return web.json_response({"results": results})
 
 
 @docs(tags=["connection"], summary="Fetch a single connection record")
-@match_info_schema(ConnIdMatchInfoSchema())
-@response_schema(ConnectionRecordSchema(), 200)
+@match_info_schema(ConnectionsConnIdMatchInfoSchema())
+@response_schema(ConnRecordSchema(), 200, description="")
 async def connections_retrieve(request: web.BaseRequest):
     """
     Request handler for fetching a single connection record.
@@ -282,11 +388,13 @@ async def connections_retrieve(request: web.BaseRequest):
         The connection record response
 
     """
-    context = request.app["request_context"]
+    context: AdminRequestContext = request["context"]
     connection_id = request.match_info["conn_id"]
 
+    profile = context.profile
     try:
-        record = await ConnectionRecord.retrieve_by_id(context, connection_id)
+        async with profile.session() as session:
+            record = await ConnRecord.retrieve_by_id(session, connection_id)
         result = record.serialize()
     except StorageNotFoundError as err:
         raise web.HTTPNotFound(reason=err.roll_up) from err
@@ -296,12 +404,93 @@ async def connections_retrieve(request: web.BaseRequest):
     return web.json_response(result)
 
 
+@docs(tags=["connection"], summary="Fetch connection remote endpoint")
+@match_info_schema(ConnectionsConnIdMatchInfoSchema())
+@response_schema(EndpointsResultSchema(), 200, description="")
+async def connections_endpoints(request: web.BaseRequest):
+    """
+    Request handler for fetching connection endpoints.
+
+    Args:
+        request: aiohttp request object
+
+    Returns:
+        The endpoints response
+
+    """
+    context: AdminRequestContext = request["context"]
+    connection_id = request.match_info["conn_id"]
+
+    profile = context.profile
+    connection_mgr = ConnectionManager(profile)
+    try:
+        endpoints = await connection_mgr.get_endpoints(connection_id)
+    except StorageNotFoundError as err:
+        raise web.HTTPNotFound(reason=err.roll_up) from err
+    except (BaseModelError, StorageError, WalletError) as err:
+        raise web.HTTPBadRequest(reason=err.roll_up) from err
+
+    return web.json_response(dict(zip(("my_endpoint", "their_endpoint"), endpoints)))
+
+
+@docs(tags=["connection"], summary="Fetch connection metadata")
+@match_info_schema(ConnectionsConnIdMatchInfoSchema())
+@querystring_schema(ConnectionMetadataQuerySchema())
+@response_schema(ConnectionMetadataSchema(), 200, description="")
+async def connections_metadata(request: web.BaseRequest):
+    """Handle fetching metadata associated with a single connection record."""
+    context: AdminRequestContext = request["context"]
+    connection_id = request.match_info["conn_id"]
+    key = request.query.get("key", None)
+
+    profile = context.profile
+    try:
+        async with profile.session() as session:
+            record = await ConnRecord.retrieve_by_id(session, connection_id)
+            if key:
+                result = await record.metadata_get(session, key)
+            else:
+                result = await record.metadata_get_all(session)
+    except StorageNotFoundError as err:
+        raise web.HTTPNotFound(reason=err.roll_up) from err
+    except BaseModelError as err:
+        raise web.HTTPBadRequest(reason=err.roll_up) from err
+
+    return web.json_response({"results": result})
+
+
+@docs(tags=["connection"], summary="Set connection metadata")
+@match_info_schema(ConnectionsConnIdMatchInfoSchema())
+@request_schema(ConnectionMetadataSetRequestSchema())
+@response_schema(ConnectionMetadataSchema(), 200, description="")
+async def connections_metadata_set(request: web.BaseRequest):
+    """Handle fetching metadata associated with a single connection record."""
+    context: AdminRequestContext = request["context"]
+    connection_id = request.match_info["conn_id"]
+    body = await request.json() if request.body_exists else {}
+
+    profile = context.profile
+    try:
+        async with profile.session() as session:
+            record = await ConnRecord.retrieve_by_id(session, connection_id)
+            for key, value in body.get("metadata", {}).items():
+                await record.metadata_set(session, key, value)
+            result = await record.metadata_get_all(session)
+    except StorageNotFoundError as err:
+        raise web.HTTPNotFound(reason=err.roll_up) from err
+    except BaseModelError as err:
+        raise web.HTTPBadRequest(reason=err.roll_up) from err
+
+    return web.json_response({"results": result})
+
+
 @docs(
     tags=["connection"],
     summary="Create a new connection invitation",
 )
 @querystring_schema(CreateInvitationQueryStringSchema())
-@response_schema(InvitationResultSchema(), 200)
+@request_schema(CreateInvitationRequestSchema())
+@response_schema(InvitationResultSchema(), 200, description="")
 async def connections_create_invitation(request: web.BaseRequest):
     """
     Request handler for creating a new connection invitation.
@@ -313,22 +502,39 @@ async def connections_create_invitation(request: web.BaseRequest):
         The connection invitation details
 
     """
-    context = request.app["request_context"]
+    context: AdminRequestContext = request["context"]
     auto_accept = json.loads(request.query.get("auto_accept", "null"))
     alias = request.query.get("alias")
     public = json.loads(request.query.get("public", "false"))
     multi_use = json.loads(request.query.get("multi_use", "false"))
+    body = await request.json() if request.body_exists else {}
+    my_label = body.get("my_label")
+    recipient_keys = body.get("recipient_keys")
+    service_endpoint = body.get("service_endpoint")
+    routing_keys = body.get("routing_keys")
+    metadata = body.get("metadata")
+    mediation_id = body.get("mediation_id")
 
     if public and not context.settings.get("public_invites"):
         raise web.HTTPForbidden(
             reason="Configuration does not include public invitations"
         )
-    base_url = context.settings.get("invite_base_url")
+    profile = context.profile
+    base_url = profile.settings.get("invite_base_url")
 
-    connection_mgr = ConnectionManager(context)
+    connection_mgr = ConnectionManager(profile)
     try:
         (connection, invitation) = await connection_mgr.create_invitation(
-            auto_accept=auto_accept, public=public, multi_use=multi_use, alias=alias
+            my_label=my_label,
+            auto_accept=auto_accept,
+            public=public,
+            multi_use=multi_use,
+            alias=alias,
+            recipient_keys=recipient_keys,
+            my_endpoint=service_endpoint,
+            routing_keys=routing_keys,
+            metadata=metadata,
+            mediation_id=mediation_id,
         )
 
         result = {
@@ -336,7 +542,7 @@ async def connections_create_invitation(request: web.BaseRequest):
             "invitation": invitation.serialize(),
             "invitation_url": invitation.to_url(base_url),
         }
-    except (ConnectionManagerError, BaseModelError) as err:
+    except (ConnectionManagerError, StorageError, BaseModelError) as err:
         raise web.HTTPBadRequest(reason=err.roll_up) from err
 
     if connection and connection.alias:
@@ -351,7 +557,7 @@ async def connections_create_invitation(request: web.BaseRequest):
 )
 @querystring_schema(ReceiveInvitationQueryStringSchema())
 @request_schema(ReceiveInvitationRequestSchema())
-@response_schema(ConnectionRecordSchema(), 200)
+@response_schema(ConnRecordSchema(), 200, description="")
 async def connections_receive_invitation(request: web.BaseRequest):
     """
     Request handler for receiving a new connection invitation.
@@ -363,20 +569,22 @@ async def connections_receive_invitation(request: web.BaseRequest):
         The resulting connection record details
 
     """
-    context = request.app["request_context"]
+    context: AdminRequestContext = request["context"]
     if context.settings.get("admin.no_receive_invites"):
         raise web.HTTPForbidden(
             reason="Configuration does not allow receipt of invitations"
         )
-    connection_mgr = ConnectionManager(context)
+    profile = context.profile
+    connection_mgr = ConnectionManager(profile)
     invitation_json = await request.json()
 
     try:
         invitation = ConnectionInvitation.deserialize(invitation_json)
         auto_accept = json.loads(request.query.get("auto_accept", "null"))
         alias = request.query.get("alias")
+        mediation_id = request.query.get("mediation_id")
         connection = await connection_mgr.receive_invitation(
-            invitation, auto_accept=auto_accept, alias=alias
+            invitation, auto_accept=auto_accept, alias=alias, mediation_id=mediation_id
         )
         result = connection.serialize()
     except (ConnectionManagerError, StorageError, BaseModelError) as err:
@@ -389,9 +597,9 @@ async def connections_receive_invitation(request: web.BaseRequest):
     tags=["connection"],
     summary="Accept a stored connection invitation",
 )
-@match_info_schema(ConnIdMatchInfoSchema())
+@match_info_schema(ConnectionsConnIdMatchInfoSchema())
 @querystring_schema(AcceptInvitationQueryStringSchema())
-@response_schema(ConnectionRecordSchema(), 200)
+@response_schema(ConnRecordSchema(), 200, description="")
 async def connections_accept_invitation(request: web.BaseRequest):
     """
     Request handler for accepting a stored connection invitation.
@@ -403,16 +611,29 @@ async def connections_accept_invitation(request: web.BaseRequest):
         The resulting connection record details
 
     """
-    context = request.app["request_context"]
-    outbound_handler = request.app["outbound_message_router"]
+    context: AdminRequestContext = request["context"]
+    outbound_handler = request["outbound_message_router"]
     connection_id = request.match_info["conn_id"]
+    profile = context.profile
 
     try:
-        connection = await ConnectionRecord.retrieve_by_id(context, connection_id)
-        connection_mgr = ConnectionManager(context)
-        my_label = request.query.get("my_label") or None
-        my_endpoint = request.query.get("my_endpoint") or None
-        request = await connection_mgr.create_request(connection, my_label, my_endpoint)
+        async with profile.session() as session:
+            connection = await ConnRecord.retrieve_by_id(session, connection_id)
+        connection_mgr = ConnectionManager(profile)
+        my_label = request.query.get("my_label")
+        my_endpoint = request.query.get("my_endpoint")
+        mediation_id = request.query.get("mediation_id")
+
+        try:
+            request = await connection_mgr.create_request(
+                connection, my_label, my_endpoint, mediation_id=mediation_id
+            )
+        except StorageError as err:
+            # Handle storage errors (including not found errors) from
+            # create_request separately as these errors represent a bad request
+            # rather than a bad url
+            raise web.HTTPBadRequest(reason=err.roll_up) from err
+
         result = connection.serialize()
     except StorageNotFoundError as err:
         raise web.HTTPNotFound(reason=err.roll_up) from err
@@ -427,9 +648,9 @@ async def connections_accept_invitation(request: web.BaseRequest):
     tags=["connection"],
     summary="Accept a stored connection request",
 )
-@match_info_schema(ConnIdMatchInfoSchema())
+@match_info_schema(ConnectionsConnIdMatchInfoSchema())
 @querystring_schema(AcceptRequestQueryStringSchema())
-@response_schema(ConnectionRecordSchema(), 200)
+@response_schema(ConnRecordSchema(), 200, description="")
 async def connections_accept_request(request: web.BaseRequest):
     """
     Request handler for accepting a stored connection request.
@@ -441,13 +662,15 @@ async def connections_accept_request(request: web.BaseRequest):
         The resulting connection record details
 
     """
-    context = request.app["request_context"]
-    outbound_handler = request.app["outbound_message_router"]
+    context: AdminRequestContext = request["context"]
+    outbound_handler = request["outbound_message_router"]
     connection_id = request.match_info["conn_id"]
 
+    profile = context.profile
     try:
-        connection = await ConnectionRecord.retrieve_by_id(context, connection_id)
-        connection_mgr = ConnectionManager(context)
+        async with profile.session() as session:
+            connection = await ConnRecord.retrieve_by_id(session, connection_id)
+        connection_mgr = ConnectionManager(profile)
         my_endpoint = request.query.get("my_endpoint") or None
         response = await connection_mgr.create_response(connection, my_endpoint)
         result = connection.serialize()
@@ -464,6 +687,7 @@ async def connections_accept_request(request: web.BaseRequest):
     tags=["connection"], summary="Assign another connection as the inbound connection"
 )
 @match_info_schema(ConnIdRefIdMatchInfoSchema())
+@response_schema(ConnectionModuleResponseSchema(), 200, description="")
 async def connections_establish_inbound(request: web.BaseRequest):
     """
     Request handler for setting the inbound connection on a connection record.
@@ -471,14 +695,16 @@ async def connections_establish_inbound(request: web.BaseRequest):
     Args:
         request: aiohttp request object
     """
-    context = request.app["request_context"]
+    context: AdminRequestContext = request["context"]
     connection_id = request.match_info["conn_id"]
-    outbound_handler = request.app["outbound_message_router"]
+    outbound_handler = request["outbound_message_router"]
     inbound_connection_id = request.match_info["ref_id"]
 
+    profile = context.profile
     try:
-        connection = await ConnectionRecord.retrieve_by_id(context, connection_id)
-        connection_mgr = ConnectionManager(context)
+        async with profile.session() as session:
+            connection = await ConnRecord.retrieve_by_id(session, connection_id)
+        connection_mgr = ConnectionManager(profile)
         await connection_mgr.establish_inbound(
             connection, inbound_connection_id, outbound_handler
         )
@@ -491,7 +717,8 @@ async def connections_establish_inbound(request: web.BaseRequest):
 
 
 @docs(tags=["connection"], summary="Remove an existing connection record")
-@match_info_schema(ConnIdMatchInfoSchema())
+@match_info_schema(ConnectionsConnIdMatchInfoSchema())
+@response_schema(ConnectionModuleResponseSchema, 200, description="")
 async def connections_remove(request: web.BaseRequest):
     """
     Request handler for removing a connection record.
@@ -499,12 +726,14 @@ async def connections_remove(request: web.BaseRequest):
     Args:
         request: aiohttp request object
     """
-    context = request.app["request_context"]
+    context: AdminRequestContext = request["context"]
     connection_id = request.match_info["conn_id"]
+    profile = context.profile
 
     try:
-        connection = await ConnectionRecord.retrieve_by_id(context, connection_id)
-        await connection.delete_record(context)
+        async with profile.session() as session:
+            connection = await ConnRecord.retrieve_by_id(session, connection_id)
+            await connection.delete_record(session)
     except StorageNotFoundError as err:
         raise web.HTTPNotFound(reason=err.roll_up) from err
     except StorageError as err:
@@ -515,7 +744,7 @@ async def connections_remove(request: web.BaseRequest):
 
 @docs(tags=["connection"], summary="Create a new static connection")
 @request_schema(ConnectionStaticRequestSchema())
-@response_schema(ConnectionStaticResultSchema(), 200)
+@response_schema(ConnectionStaticResultSchema(), 200, description="")
 async def connections_create_static(request: web.BaseRequest):
     """
     Request handler for creating a new static connection.
@@ -527,10 +756,11 @@ async def connections_create_static(request: web.BaseRequest):
         The new connection record
 
     """
-    context = request.app["request_context"]
+    context: AdminRequestContext = request["context"]
     body = await request.json()
 
-    connection_mgr = ConnectionManager(context)
+    profile = context.profile
+    connection_mgr = ConnectionManager(profile)
     try:
         (
             my_info,
@@ -543,7 +773,6 @@ async def connections_create_static(request: web.BaseRequest):
             their_did=body.get("their_did") or None,
             their_verkey=body.get("their_verkey") or None,
             their_endpoint=body.get("their_endpoint") or None,
-            their_role=body.get("their_role") or None,
             their_label=body.get("their_label") or None,
             alias=body.get("alias") or None,
         )
@@ -568,6 +797,17 @@ async def register(app: web.Application):
         [
             web.get("/connections", connections_list, allow_head=False),
             web.get("/connections/{conn_id}", connections_retrieve, allow_head=False),
+            web.get(
+                "/connections/{conn_id}/metadata",
+                connections_metadata,
+                allow_head=False,
+            ),
+            web.post("/connections/{conn_id}/metadata", connections_metadata_set),
+            web.get(
+                "/connections/{conn_id}/endpoints",
+                connections_endpoints,
+                allow_head=False,
+            ),
             web.post("/connections/create-static", connections_create_static),
             web.post("/connections/create-invitation", connections_create_invitation),
             web.post("/connections/receive-invitation", connections_receive_invitation),
@@ -576,7 +816,8 @@ async def register(app: web.Application):
                 connections_accept_invitation,
             ),
             web.post(
-                "/connections/{conn_id}/accept-request", connections_accept_request
+                "/connections/{conn_id}/accept-request",
+                connections_accept_request,
             ),
             web.post(
                 "/connections/{conn_id}/establish-inbound/{ref_id}",
