@@ -1,8 +1,8 @@
 """Represents an explicit RFC 15 ack message, adopted into present-proof protocol."""
 
-from marshmallow import EXCLUDE
+from marshmallow import EXCLUDE, fields, validate
 
-from .....messaging.ack.message import Ack, AckSchema
+from ....notification.v1_0.messages.ack import V10Ack, V10AckSchema
 
 from ..message_types import PRESENTATION_ACK, PROTOCOL_PACKAGE
 
@@ -11,7 +11,7 @@ HANDLER_CLASS = (
 )
 
 
-class PresentationAck(Ack):
+class PresentationAck(V10Ack):
     """Base class representing an explicit ack message for present-proof protocol."""
 
     class Meta:
@@ -21,7 +21,7 @@ class PresentationAck(Ack):
         message_type = PRESENTATION_ACK
         schema_class = "PresentationAckSchema"
 
-    def __init__(self, status: str = None, **kwargs):
+    def __init__(self, status: str = None, verification_result: str = None, **kwargs):
         """
         Initialize an explicit ack message instance.
 
@@ -30,9 +30,10 @@ class PresentationAck(Ack):
 
         """
         super().__init__(status, **kwargs)
+        self._verification_result = verification_result
 
 
-class PresentationAckSchema(AckSchema):
+class PresentationAckSchema(V10AckSchema):
     """Schema for PresentationAck class."""
 
     class Meta:
@@ -40,3 +41,10 @@ class PresentationAckSchema(AckSchema):
 
         model_class = PresentationAck
         unknown = EXCLUDE
+
+    verification_result = fields.Str(
+        required=False,
+        description="Whether presentation is verified: true or false",
+        example="true",
+        validate=validate.OneOf(["true", "false"]),
+    )
