@@ -3,11 +3,11 @@
 from ...didcomm_prefix import DIDCommPrefix
 
 # Message types
-FORWARD = f"routing/1.0/forward"
-ROUTE_QUERY_REQUEST = f"routing/1.0/route-query-request"
-ROUTE_QUERY_RESPONSE = f"routing/1.0/route-query-response"
-ROUTE_UPDATE_REQUEST = f"routing/1.0/route-update-request"
-ROUTE_UPDATE_RESPONSE = f"routing/1.0/route-update-response"
+FORWARD = "routing/1.0/forward"
+ROUTE_QUERY_REQUEST = "routing/1.0/route-query-request"
+ROUTE_QUERY_RESPONSE = "routing/1.0/route-query-response"
+ROUTE_UPDATE_REQUEST = "routing/1.0/route-update-request"
+ROUTE_UPDATE_RESPONSE = "routing/1.0/route-update-response"
 
 PROTOCOL_PACKAGE = "aries_cloudagent.protocols.routing.v1_0"
 
